unit xmlconftest;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testregistry, xmlconf;

type
  TTestXMLConfig = class(TTestCase)
  private
    function  CreateConf(AFileName: String; AStartEmpty: Boolean = False): TXMLConfig;
    procedure DeleteConf(C: TXMLConfig; DeleteConfFile: Boolean = true);
  published
    procedure TestConcurrency;
  end;


implementation


function TTestXMLConfig.CreateConf(AFileName: String; AStartEmpty: Boolean): TXMLConfig;
begin
  Result := TXMLConfig.Create(nil);
  Result.StartEmpty := AStartEmpty;
  Result.FileName := AFileName;
end;

procedure TTestXMLConfig.DeleteConf(C: TXMLConfig; DeleteConfFile: Boolean);
var
  FN: String;
begin
  if DeleteConfFile then
    FN := C.FileName;
  FreeAndNil(C);
  if DeleteConfFile then
    DeleteFile(FN);
end;

procedure TTestXMLConfig.TestConcurrency;
const
  form1 = 'ReportForm/';
  form2 = 'LearnerListForm/';
var
  Co1, Co2: TXMLConfig;
begin
  // form 1
  Co1 := CreateConf('concurrency.xml');
  Co1.SetValue(form1+'Width', 400);
  Co1.SetValue(form1+'Height', 300);
  Co1.SetValue(form1+'Maximized', True);

  // form 2
  Co2 := CreateConf('concurrency.xml');
  Co2.SetValue(form2+'Width', 456);
  Co2.SetValue(form2+'Height', 600);
  Co2.SetValue(form2+'Maximized', False);

  // Simulate Form 1 closing
  DeleteConf(Co1, False);

  // Simulate Form 2 closing
  DeleteConf(Co2, False);

  // Now load Form 1 again
  Co1 := CreateConf('concurrency.xml');
  try
    AssertEquals('Failed on 1', 400, Co1.GetValue(form1+'Width', 100));
    AssertEquals('Failed on 2', 300, Co1.GetValue(form1+'Height', 100));
    AssertEquals('Failed on 3', True, Co1.GetValue(form1+'Maximized', False));
  finally
    // Test cleanup
    DeleteConf(Co1, False);
  end;
end;


initialization
  RegisterTest(TTestXMLConfig);

end.

